@@ -189,11 +189,11 @@ gen8_render_set_surface_state(
 
     memset(ss, 0, sizeof(*ss));
 
-    switch (flags & (I965_PP_FLAG_TOP_FIELD|I965_PP_FLAG_BOTTOM_FIELD)) {
-    case I965_PP_FLAG_BOTTOM_FIELD:
+    switch (flags & (VA_TOP_FIELD|VA_BOTTOM_FIELD)) {
+    case VA_BOTTOM_FIELD:
         ss->ss0.vert_line_stride_ofs = 1;
         /* fall-through */
-    case I965_PP_FLAG_TOP_FIELD:
+    case VA_TOP_FIELD:
         ss->ss0.vert_line_stride = 1;
         height /= 2;
         break;
@@ -276,6 +276,9 @@ gen8_render_src_surfaces_state(
     gen8_render_src_surface_state(ctx, 1, region, 0, rw, rh, region_pitch, I965_SURFACEFORMAT_R8_UNORM, flags);     /* Y */
     gen8_render_src_surface_state(ctx, 2, region, 0, rw, rh, region_pitch, I965_SURFACEFORMAT_R8_UNORM, flags);
 
+    if (obj_surface->fourcc == VA_FOURCC_Y800) /* single plane for grayscale */
+        return;
+
     if (obj_surface->fourcc == VA_FOURCC_NV12) {
         gen8_render_src_surface_state(ctx, 3, region,
                                       region_pitch * obj_surface->y_cb_offset,
@@ -947,6 +950,7 @@ gen8_emit_vertex_element_state(VADriverContextP ctx)
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx);
     struct intel_batchbuffer *batch = i965->batch;
+    int i;
 
     /*
      * The VUE layout
@@ -990,6 +994,17 @@ gen8_emit_vertex_element_state(VADriverContextP ctx)
               (I965_VFCOMPONENT_STORE_SRC << VE1_VFCOMPONENT_1_SHIFT) |
               (I965_VFCOMPONENT_STORE_1_FLT << VE1_VFCOMPONENT_2_SHIFT) |
               (I965_VFCOMPONENT_STORE_1_FLT << VE1_VFCOMPONENT_3_SHIFT));
+
+    /* Disable instancing for all vertex elements. */
+    for (i = 0; i < 3; i++) {
+	OUT_BATCH(batch, GEN8_3DSTATE_VF_INSTANCING | (3 - 2));
+	OUT_BATCH(batch, i);
+	OUT_BATCH(batch, 0);
+    }
+
+    /* Disable system-generated values. */
+    OUT_BATCH(batch, GEN8_3DSTATE_VF_SGVS | (2 - 2));
+    OUT_BATCH(batch, 0);
 }
 
 static void
@@ -1377,7 +1392,7 @@ gen8_emit_wm_state(VADriverContextP ctx, int kernel)
     unsigned int num_samples = 0;
     unsigned int max_threads;
 
-    max_threads = render_state->max_wm_threads - 2;
+    max_threads = i965->intel.device_info->max_wm_threads - 2;
 
     BEGIN_BATCH(batch, 2);
     OUT_BATCH(batch, GEN8_3DSTATE_PSEXTRA | (2 - 2));
@@ -1436,7 +1451,8 @@ gen8_emit_wm_state(VADriverContextP ctx, int kernel)
     /* DW3. PS shader flag .Binding table cnt/sample cnt */
     OUT_BATCH(batch,
               (1 << GEN7_PS_SAMPLER_COUNT_SHIFT) |
-              (5 << GEN7_PS_BINDING_TABLE_ENTRY_COUNT_SHIFT));
+              (5 << GEN7_PS_BINDING_TABLE_ENTRY_COUNT_SHIFT) |
+              GEN7_PS_VECTOR_MASK_ENABLE);
     /* DW4-5. Scatch space */
     OUT_BATCH(batch, 0); /* scratch space base offset */
     OUT_BATCH(batch, 0);
@@ -1721,6 +1737,40 @@ gen8_render_put_subpicture(
     intel_batchbuffer_flush(batch);
 }
 
+static void
+gen8_render_terminate(VADriverContextP ctx)
+{
+    struct i965_driver_data *i965 = i965_driver_data(ctx);
+    struct i965_render_state *render_state = &i965->render_state;
+
+    dri_bo_unreference(render_state->vb.vertex_buffer);
+    render_state->vb.vertex_buffer = NULL;
+
+    dri_bo_unreference(render_state->wm.surface_state_binding_table_bo);
+    render_state->wm.surface_state_binding_table_bo = NULL;
+
+    if (render_state->instruction_state.bo) {
+        dri_bo_unreference(render_state->instruction_state.bo);
+        render_state->instruction_state.bo = NULL;
+    }
+
+    if (render_state->dynamic_state.bo) {
+        dri_bo_unreference(render_state->dynamic_state.bo);
+        render_state->dynamic_state.bo = NULL;
+    }
+
+    if (render_state->indirect_state.bo) {
+        dri_bo_unreference(render_state->indirect_state.bo);
+        render_state->indirect_state.bo = NULL;
+    }
+
+    if (render_state->draw_region) {
+        dri_bo_unreference(render_state->draw_region->bo);
+        free(render_state->draw_region);
+        render_state->draw_region = NULL;
+    }
+}
+
 bool
 gen8_render_init(VADriverContextP ctx)
 {
@@ -1733,11 +1783,10 @@ gen8_render_init(VADriverContextP ctx)
 
     render_state->render_put_surface = gen8_render_put_surface;
     render_state->render_put_subpicture = gen8_render_put_subpicture;
+    render_state->render_terminate = gen8_render_terminate;
 
-    if (IS_GEN8(i965->intel.device_id)) {
-        memcpy(render_state->render_kernels, render_kernels_gen8,
-			sizeof(render_state->render_kernels));
-    }
+    memcpy(render_state->render_kernels, render_kernels_gen8,
+           sizeof(render_state->render_kernels));
 
     kernel_size = 4096;
 
@@ -1784,49 +1833,5 @@ gen8_render_init(VADriverContextP ctx)
 
     dri_bo_unmap(render_state->instruction_state.bo);
 
-
-    if (IS_GEN8(i965->intel.device_id)) {
-        render_state->max_wm_threads = 64;
-    } else {
-        /* should never get here !!! */
-        assert(0);
-    }
-
     return true;
 }
-
-
-void
-gen8_render_terminate(VADriverContextP ctx)
-{
-    struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct i965_render_state *render_state = &i965->render_state;
-
-    dri_bo_unreference(render_state->vb.vertex_buffer);
-    render_state->vb.vertex_buffer = NULL;
-
-    dri_bo_unreference(render_state->wm.surface_state_binding_table_bo);
-    render_state->wm.surface_state_binding_table_bo = NULL;
-
-    if (render_state->instruction_state.bo) {
-        dri_bo_unreference(render_state->instruction_state.bo);
-        render_state->instruction_state.bo = NULL;
-    }
-
-    if (render_state->dynamic_state.bo) {
-        dri_bo_unreference(render_state->dynamic_state.bo);
-        render_state->dynamic_state.bo = NULL;
-    }
-
-    if (render_state->indirect_state.bo) {
-        dri_bo_unreference(render_state->indirect_state.bo);
-        render_state->indirect_state.bo = NULL;
-    }
-
-    if (render_state->draw_region) {
-        dri_bo_unreference(render_state->draw_region->bo);
-        free(render_state->draw_region);
-        render_state->draw_region = NULL;
-    }
-}
-
