@@ -106,6 +106,11 @@
 # define GEN6_3DSTATE_SF_LINE_PROVOKE_SHIFT		27
 # define GEN6_3DSTATE_SF_TRIFAN_PROVOKE_SHIFT		25
 
+#define GEN8_3DSTATE_RASTER			CMD(3, 0, 0x50)
+# define GEN8_3DSTATE_RASTER_CULL_BOTH			(0 << 16)
+# define GEN8_3DSTATE_RASTER_CULL_NONE			(1 << 16)
+# define GEN8_3DSTATE_RASTER_CULL_FRONT			(2 << 16)
+# define GEN8_3DSTATE_RASTER_CULL_BACK			(3 << 16)
 
 #define GEN6_3DSTATE_WM				CMD(3, 0, 0x14)
 /* DW2 */
@@ -173,6 +178,10 @@
 #define GEN6_3DSTATE_CONSTANT_GS          	CMD(3, 0, 0x16)
 #define GEN6_3DSTATE_CONSTANT_PS          	CMD(3, 0, 0x17)
 
+/* Gen8 WM_HZ_OP */
+#define GEN8_3DSTATE_WM_HZ_OP			CMD(3, 0, 0x52)
+
+
 # define GEN6_3DSTATE_CONSTANT_BUFFER_3_ENABLE  (1 << 15)
 # define GEN6_3DSTATE_CONSTANT_BUFFER_2_ENABLE  (1 << 14)
 # define GEN6_3DSTATE_CONSTANT_BUFFER_1_ENABLE  (1 << 13)
@@ -188,9 +197,13 @@
 # define GEN6_3DSTATE_MULTISAMPLE_NUMSAMPLES_4                  (2 << 1)
 # define GEN6_3DSTATE_MULTISAMPLE_NUMSAMPLES_8                  (3 << 1)
 
+#define GEN8_3DSTATE_MULTISAMPLE		CMD(3, 0, 0x0d)
+#define GEN8_3DSTATE_SAMPLE_PATTERN		CMD(3, 1, 0x1C)
+
 /* GEN7 */
 #define GEN7_3DSTATE_CLEAR_PARAMS               CMD(3, 0, 0x04)
 #define GEN7_3DSTATE_DEPTH_BUFFER               CMD(3, 0, 0x05)
+#define GEN7_3DSTATE_HIER_DEPTH_BUFFER		CMD(3, 0, 0x07)
 
 #define GEN7_3DSTATE_URB_VS                     CMD(3, 0, 0x30)
 #define GEN7_3DSTATE_URB_HS                     CMD(3, 0, 0x31)
@@ -203,8 +216,14 @@
 
 #define GEN7_3DSTATE_PUSH_CONSTANT_ALLOC_VS     CMD(3, 1, 0x12)
 #define GEN7_3DSTATE_PUSH_CONSTANT_ALLOC_PS     CMD(3, 1, 0x16)
+
+#define GEN7_3DSTATE_PUSH_CONSTANT_ALLOC_DS     CMD(3, 1, 0x14)
+#define GEN7_3DSTATE_PUSH_CONSTANT_ALLOC_HS     CMD(3, 1, 0x13)
+#define GEN7_3DSTATE_PUSH_CONSTANT_ALLOC_GS     CMD(3, 1, 0x15)
 /* DW1 */
 # define GEN7_PUSH_CONSTANT_BUFFER_OFFSET_SHIFT 16
+# define GEN8_PUSH_CONSTANT_BUFFER_OFFSET_SHIFT	16
+# define GEN8_PUSH_CONSTANT_BUFFER_SIZE_SHIFT	0
 
 #define GEN7_3DSTATE_CONSTANT_HS                CMD(3, 0, 0x19)
 #define GEN7_3DSTATE_CONSTANT_DS                CMD(3, 0, 0x1a)
@@ -222,6 +241,11 @@
 # define GEN7_SBE_POINT_SPRITE_LOWERLEFT        (1 << 20)
 # define GEN7_SBE_URB_ENTRY_READ_LENGTH_SHIFT   11
 # define GEN7_SBE_URB_ENTRY_READ_OFFSET_SHIFT   4
+# define GEN8_SBE_FORCE_URB_ENTRY_READ_LENGTH  (1 << 29)
+# define GEN8_SBE_FORCE_URB_ENTRY_READ_OFFSET  (1 << 28)
+
+# define GEN8_SBE_URB_ENTRY_READ_OFFSET_SHIFT   5
+#define GEN8_3DSTATE_SBE_SWIZ                    CMD(3, 0, 0x51)
 
 #define GEN7_3DSTATE_PS                                 CMD(3, 0, 0x20)
 /* DW1: kernel pointer */
@@ -254,6 +278,37 @@
 /* DW6: kernel 1 pointer */
 /* DW7: kernel 2 pointer */
 
+# define GEN8_PS_MAX_THREADS_SHIFT                      23
+
+#define GEN8_3DSTATE_PSEXTRA				CMD(3, 0, 0x4f)
+/* DW1 */
+# define GEN8_PSX_PIXEL_SHADER_VALID                    (1 << 31)
+# define GEN8_PSX_PSCDEPTH_OFF                          (0 << 26)
+# define GEN8_PSX_PSCDEPTH_ON                           (1 << 26)
+# define GEN8_PSX_PSCDEPTH_ON_GE                        (2 << 26)
+# define GEN8_PSX_PSCDEPTH_ON_LE                        (3 << 26)
+# define GEN8_PSX_ATTRIBUTE_ENABLE			(1 << 8)
+
+#define GEN8_3DSTATE_PSBLEND				CMD(3, 0, 0x4d)
+/* DW1 */
+# define GEN8_PS_BLEND_ALPHA_TO_COVERAGE_ENABLE         (1 << 31)
+# define GEN8_PS_BLEND_HAS_WRITEABLE_RT                 (1 << 30)
+# define GEN8_PS_BLEND_COLOR_BUFFER_BLEND_ENABLE        (1 << 29)
+# define GEN8_PS_BLEND_SRC_ALPHA_BLEND_FACTOR_MASK      INTEL_MASK(28, 24)
+# define GEN8_PS_BLEND_SRC_ALPHA_BLEND_FACTOR_SHIFT     24
+# define GEN8_PS_BLEND_DST_ALPHA_BLEND_FACTOR_MASK      INTEL_MASK(23, 19)
+# define GEN8_PS_BLEND_DST_ALPHA_BLEND_FACTOR_SHIFT     19
+# define GEN8_PS_BLEND_SRC_BLEND_FACTOR_MASK            INTEL_MASK(18, 14)
+# define GEN8_PS_BLEND_SRC_BLEND_FACTOR_SHIFT           14
+# define GEN8_PS_BLEND_DST_BLEND_FACTOR_MASK            INTEL_MASK(13, 9)
+# define GEN8_PS_BLEND_DST_BLEND_FACTOR_SHIFT           9
+# define GEN8_PS_BLEND_ALPHA_TEST_ENABLE                (1 << 8)
+# define GEN8_PS_BLEND_INDEPENDENT_ALPHA_BLEND_ENABLE   (1 << 7)
+
+
+#define GEN7_3DSTATE_STENCIL_BUFFER			CMD(3, 0, 0x06)
+#define GEN8_3DSTATE_WM_DEPTH_STENCIL			CMD(3, 0, 0x4e)
+
 #define GEN7_3DSTATE_VIEWPORT_STATE_POINTERS_SF_CL      CMD(3, 0, 0x21)
 #define GEN7_3DSTATE_VIEWPORT_STATE_POINTERS_CC         CMD(3, 0, 0x23)
 
@@ -269,6 +324,14 @@
 #define GEN7_3DSTATE_SAMPLER_STATE_POINTERS_VS          CMD(3, 0, 0x2b)
 #define GEN7_3DSTATE_SAMPLER_STATE_POINTERS_GS          CMD(3, 0, 0x2e)
 #define GEN7_3DSTATE_SAMPLER_STATE_POINTERS_PS          CMD(3, 0, 0x2f)
+#define GEN7_3DSTATE_SAMPLER_STATE_POINTERS_HS          CMD(3, 0, 0x2c)
+#define GEN7_3DSTATE_SAMPLER_STATE_POINTERS_DS          CMD(3, 0, 0x2d)
+
+#define GEN7_3DSTATE_VF                                 CMD(3, 0, 0x0c)
+#define GEN8_3DSTATE_VF_INSTANCING                      CMD(3, 0, 0x49)
+
+#define GEN7_3DSTATE_VF                                 CMD(3, 0, 0x0c)
+#define GEN8_3DSTATE_VF_INSTANCING                      CMD(3, 0, 0x49)
 
 #define MFX(pipeline, op, sub_opa, sub_opb)     \
     (3 << 29 |                                  \
@@ -322,11 +385,26 @@
 
 #define MFD_VC1_BSD_OBJECT                      MFX(2, 2, 1, 8)
 
+//MFX_JPEG #defines for JPEG decoder only
 #define MFX_JPEG_PIC_STATE                      MFX(2, 7, 0, 0)
 #define MFX_JPEG_HUFF_TABLE_STATE               MFX(2, 7, 0, 2)
 
+//MFC_JPEG #defines for JPEG encoder only
+#define MFC_JPEG_SCAN_OBJECT                    MFX(2, 7, 2, 9)
+#define MFC_JPEG_HUFF_TABLE_STATE               MFX(2, 7, 2, 3)
+
 #define MFD_JPEG_BSD_OBJECT                     MFX(2, 7, 1, 8)
 
+#define MFX_VP8_PIC_STATE                       MFX(2, 4, 0, 0)
+
+#define MFD_VP8_BSD_OBJECT                      MFX(2, 4, 1, 8)
+
+#define MFX_VP8_ENCODER_CFG                     MFX(2, 4, 2, 1)
+
+#define MFX_VP8_BSP_BUF_BASE_ADDR_STATE         MFX(2, 4, 2, 3)
+
+#define MFX_VP8_PAK_OBJECT                      MFX(2, 4, 2, 9)
+
 #define VEB(pipeline, op, sub_opa, sub_opb)     \
      (3 << 29 |                                 \
      (pipeline) << 27 |                         \
@@ -338,6 +416,27 @@
 #define VEB_STATE                               VEB(2, 4, 0, 2)
 #define VEB_DNDI_IECP_STATE                     VEB(2, 4, 0, 3)
 
+#define HCP(command)                            \
+    (3 << 29 |                                  \
+     2 << 27 |                                  \
+     7 << 23 |                                  \
+     (command << 16))
+
+#define HCP_PIPE_MODE_SELECT                    HCP(0x00)
+#define HCP_SURFACE_STATE                       HCP(0x01)
+#define HCP_PIPE_BUF_ADDR_STATE                 HCP(0x02)
+#define HCP_IND_OBJ_BASE_ADDR_STATE             HCP(0x03)
+#define HCP_QM_STATE                            HCP(0x04)
+#define HCP_FQM_STATE                           HCP(0x05)
+#define HCP_PIC_STATE                           HCP(0x10)
+#define HCP_TILE_STATE                          HCP(0x11)
+#define HCP_REF_IDX_STATE                       HCP(0x12)
+#define HCP_WEIGHTOFFSET                        HCP(0x13)
+#define HCP_SLICE_STATE                         HCP(0x14)
+#define HCP_BSD_OBJECT                          HCP(0x20)
+#define HCP_PAK_OBJECT                          HCP(0x21)
+#define HCP_INSERT_PAK_OBJECT                   HCP(0x22)
+
 #define I965_DEPTHFORMAT_D32_FLOAT              1
 
 #define BASE_ADDRESS_MODIFY             (1 << 0)
@@ -609,6 +708,8 @@
 #define VE1_VFCOMPONENT_2_SHIFT		20
 #define VE1_VFCOMPONENT_3_SHIFT		16
 #define VE1_DESTINATION_ELEMENT_OFFSET_SHIFT	0
+#define GEN8_VE0_VERTEX_BUFFER_INDEX_SHIFT      26 /* for GEN8 */
+#define GEN8_VE0_VALID                  (1 << 25)  /* for GEN8 */
 
 #define VB0_BUFFER_INDEX_SHIFT          27
 #define GEN6_VB0_BUFFER_INDEX_SHIFT     26
@@ -618,6 +719,8 @@
 #define GEN6_VB0_INSTANCEDATA           (1 << 20)
 #define GEN7_VB0_ADDRESS_MODIFYENABLE   (1 << 14)
 #define VB0_BUFFER_PITCH_SHIFT          0
+#define GEN8_VB0_BUFFER_INDEX_SHIFT     26
+#define GEN8_VB0_MOCS_SHIFT		16
 
 #define _3DPRIMITIVE_VERTEX_SEQUENTIAL  (0 << 15)
 #define _3DPRIMITIVE_VERTEX_RANDOM      (1 << 15)
@@ -648,6 +751,10 @@
 #define _3DPRIM_LINESTRIP_CONT_BF 0x14
 #define _3DPRIM_TRIFAN_NOSTIPPLE  0x15
 
+#define GEN8_3DSTATE_VF_INSTANCING	CMD(3, 0, 0x49)
+#define GEN8_3DSTATE_VF_SGVS		CMD(3, 0, 0x4a)
+#define GEN8_3DSTATE_VF_TOPOLOGY	CMD(3, 0, 0x4b)
+
 #define I965_TILEWALK_XMAJOR                 0
 #define I965_TILEWALK_YMAJOR                 1
 
@@ -704,6 +811,8 @@
 #define MFX_FORMAT_VC1          1
 #define MFX_FORMAT_AVC          2
 #define MFX_FORMAT_JPEG         3
+#define MFX_FORMAT_SVC          4
+#define MFX_FORMAT_VP8          5
 
 #define MFX_SHORT_MODE          0
 #define MFX_LONG_MODE           1
@@ -730,9 +839,12 @@
 #define MFD_MODE_VLD            0
 #define MFD_MODE_IT             1
 
+#define MFX_SURFACE_YCRCB_NORMAL        0
+#define MFX_SURFACE_YCRCB_SWAPY         3
 #define MFX_SURFACE_PLANAR_420_8        4
 #define MFX_SURFACE_PLANAR_411_8        5
 #define MFX_SURFACE_PLANAR_422_8        6
+#define MFX_SURFACE_R8G8B8A8_UNORM      9
 #define MFX_SURFACE_MONOCHROME          12
 
 #define MPEG_I_PICTURE          1
@@ -750,10 +862,49 @@
 #define SUBSAMPLE_YUV444        4
 #define SUBSAMPLE_YUV411        5
 #define SUBSAMPLE_RGBX          6
+#define JPEG_ENC_SURFACE_NV12 1
+#define JPEG_ENC_SURFACE_UYVY 2
+#define JPEG_ENC_SURFACE_YUY2 3
+#define JPEG_ENC_SURFACE_Y8   4
+#define JPEG_ENC_SURFACE_RGB  5
 
-#define URB_SIZE(intel)         (IS_GEN7(intel->device_id) ? 4096 :     \
-                                 IS_GEN6(intel->device_id) ? 1024 :     \
-                                 IS_IRONLAKE(intel->device_id) ? 1024 : \
-                                 IS_G4X(intel->device_id) ? 384 : 256)
+#define JPEG_ENC_MCU_YUV400     0
+#define JPEG_ENC_MCU_YUV420     1
+#define JPEG_ENC_MCU_YUV422H_2Y 2
+#define JPEG_ENC_MCU_RGB        3
+
+#define JPEG_ENC_ROUND_QUANT_DEFAULT 0
+#define JPEG_ENC_ROUND_QUANT_MINUS_128TH 1
+#define JPEG_ENC_ROUND_QUANT_PLUS_128TH 2
+#define JPEG_ENC_ROUND_QUANT_MINUS_64TH 3
+#define JPEG_ENC_ROUND_QUANT_PLUS_64TH 4
+#define JPEG_ENC_ROUND_QUANT_MINUS_32TH 5
+#define JPEG_ENC_ROUND_QUANT_MINUS_16TH 6
+#define JPEG_ENC_ROUND_QUANT_MINUS_8TH 7
+
+
+
+#define GEN9_PIPELINE_SELECTION_MASK        (0x3 << 8)
+#define GEN9_FORCE_MEDIA_AWAKE_MASK         (1 << 13)
+#define GEN9_FORCE_MEDIA_AWAKE_ON           (1 << 5)
+#define GEN9_FORCE_MEDIA_AWAKE_OFF          (0 << 5)
+
+#define GEN9_MEDIA_DOP_GATE_MASK            (1 << 12)
+#define GEN9_MEDIA_DOP_GATE_ON              (1 << 4)
+#define GEN9_MEDIA_DOP_GATE_OFF             (0 << 4)
+
+#define GEN9_SBE_ACTIVE_COMPONENT_NONE          0
+#define GEN9_SBE_ACTIVE_COMPONENT_XY            1
+#define GEN9_SBE_ACTIVE_COMPONENT_XYZ           2
+#define GEN9_SBE_ACTIVE_COMPONENT_XYZW          3
+
+#define HEVC_SLICE_B                            0
+#define HEVC_SLICE_P                            1
+#define HEVC_SLICE_I                            2
+
+#define HCP_CODEC_HEVC                          0
+
+#define HCP_CODEC_SELECT_DECODE                 0
+#define HCP_CODEC_SELECT_ENCODE                 1
 
 #endif /* _I965_DEFINES_H_ */
